@@ -1,4 +1,4 @@
-# Copyright (C) 2003-2007 Nominum, Inc.
+# Copyright (C) 2003-2007, 2009 Nominum, Inc.
 #
 # Permission to use, copy, modify, and distribute this software and its
 # documentation for any purpose with or without fee is hereby granted,
@@ -157,4 +157,7 @@ def inet_aton(text):
     #
     # Finally we can go to binary.
     #
-    return text.decode('hex_codec')
+    try:
+        return text.decode('hex_codec')
+    except TypeError:
+        raise dns.exception.SyntaxError
