@@ -1,4 +1,4 @@
-# Copyright (C) 2001-2007 Nominum, Inc.
+# Copyright (C) 2001-2007, 2009 Nominum, Inc.
 #
 # Permission to use, copy, modify, and distribute this software and its
 # documentation for any purpose with or without fee is hereby granted,
@@ -25,6 +25,7 @@ import dns.exception
 import dns.flags
 import dns.name
 import dns.opcode
+import dns.entropy
 import dns.rcode
 import dns.rdata
 import dns.rdataclass
@@ -83,6 +84,8 @@ class Message(object):
     @type ednsflags: long
     @ivar payload: The EDNS payload size.  The default is 0.
     @type payload: int
+    @ivar options: The EDNS options
+    @type options: list of dns.edns.Option objects
     @ivar request_payload: The associated request's EDNS payload size.
     @type request_payload: int
     @ivar keyring: The TSIG keyring to use.  The default is None.
@@ -131,7 +134,7 @@ class Message(object):
 
     def __init__(self, id=None):
         if id is None:
-            self.id = random.randint(0, 65535)
+            self.id = dns.entropy.random_16()
         else:
             self.id = id
         self.flags = 0
@@ -142,6 +145,7 @@ class Message(object):
         self.edns = -1
         self.ednsflags = 0
         self.payload = 0
+        self.options = []
         self.request_payload = 0
         self.keyring = None
         self.keyname = None
@@ -161,7 +165,7 @@ class Message(object):
 
     def __repr__(self):
         return '<DNS message, ID ' + `self.id` + '>'
-    
+
     def __str__(self):
         return self.to_text()
 
@@ -173,7 +177,7 @@ class Message(object):
 
         @rtype: string
         """
-        
+
         s = cStringIO.StringIO()
         print >> s, 'id %d' % self.id
         print >> s, 'opcode %s' % \
@@ -288,7 +292,7 @@ class Message(object):
                    covers=dns.rdatatype.NONE, deleting=None, create=False,
                    force_unique=False):
         """Find the RRset with the given attributes in the specified section.
-        
+
         @param section: the section of the message to look in, e.g.
         self.answer.
         @type section: list of dns.rrset.RRset objects
@@ -336,7 +340,7 @@ class Message(object):
         """Get the RRset with the given attributes in the specified section.
 
         If the RRset is not found, None is returned.
-        
+
         @param section: the section of the message to look in, e.g.
         self.answer.
         @type section: list of dns.rrset.RRset objects
@@ -371,7 +375,7 @@ class Message(object):
 
         Additional keyword arguments are passed to the rrset to_wire()
         method.
-        
+
         @param origin: The origin to be appended to any relative names.
         @type origin: dns.name.Name object
         @param max_size: The maximum size of the wire format output; default
@@ -399,7 +403,7 @@ class Message(object):
         for rrset in self.authority:
             r.add_rrset(dns.renderer.AUTHORITY, rrset, **kw)
         if self.edns >= 0:
-            r.add_edns(self.edns, self.ednsflags, self.payload)
+            r.add_edns(self.edns, self.ednsflags, self.payload, self.options)
         for rrset in self.additional:
             r.add_rrset(dns.renderer.ADDITIONAL, rrset, **kw)
         r.write_header()
@@ -414,7 +418,7 @@ class Message(object):
                  tsig_error=0, other_data=''):
         """When sending, a TSIG signature using the specified keyring
         and keyname should be added.
-        
+
         @param keyring: The TSIG keyring to use; defaults to None.
         @type keyring: dict
         @param keyname: The name of the TSIG key to use; defaults to None.
@@ -449,7 +453,7 @@ class Message(object):
         self.tsig_error = tsig_error
         self.other_data = other_data
 
-    def use_edns(self, edns=0, ednsflags=0, payload=1280, request_payload=None):
+    def use_edns(self, edns=0, ednsflags=0, payload=1280, request_payload=None, options=None):
         """Configure EDNS behavior.
         @param edns: The EDNS level to use.  Specifying None, False, or -1
         means 'do not use EDNS', and in this case the other parameters are
@@ -464,6 +468,8 @@ class Message(object):
         @param request_payload: The EDNS payload size to use when sending
         this message.  If not specified, defaults to the value of payload.
         @type request_payload: int or None
+        @param options: The EDNS options
+        @type options: None or list of dns.edns.Option objects
         @see: RFC 2671
         """
         if edns is None or edns is False:
@@ -476,9 +482,17 @@ class Message(object):
             ednsflags = 0
             payload = 0
             request_payload = 0
+            options = []
+        else:
+            # make sure the EDNS version in ednsflags agrees with edns
+            ednsflags &= 0xFF00FFFFL
+            ednsflags |= (edns << 16)
+            if options is None:
+                options = []
         self.edns = edns
         self.ednsflags = ednsflags
         self.payload = payload
+        self.options = options
         self.request_payload = request_payload
 
     def want_dnssec(self, wanted=True):
@@ -509,7 +523,7 @@ class Message(object):
         (value, evalue) = dns.rcode.to_flags(rcode)
         self.flags &= 0xFFF0
         self.flags |= value
-        self.ednsflags &= 0xFF000000L
+        self.ednsflags &= 0x00FFFFFFL
         self.ednsflags |= evalue
         if self.ednsflags != 0 and self.edns < 0:
             self.edns = 0
@@ -541,19 +555,23 @@ class _WireReader(object):
     @type current: int
     @ivar updating: Is the message a dynamic update?
     @type updating: bool
+    @ivar one_rr_per_rrset: Put each RR into its own RRset?
+    @type one_rr_per_rrset: bool
     @ivar zone_rdclass: The class of the zone in messages which are
     DNS dynamic updates.
     @type zone_rdclass: int
     """
-    
-    def __init__(self, wire, message, question_only=False):
+
+    def __init__(self, wire, message, question_only=False,
+                 one_rr_per_rrset=False):
         self.wire = wire
         self.message = message
         self.current = 0
         self.updating = False
         self.zone_rdclass = dns.rdataclass.IN
         self.question_only = question_only
-        
+        self.one_rr_per_rrset = one_rr_per_rrset
+
     def _get_question(self, qcount):
         """Read the next I{qcount} records from the wire data and add them to
         the question section.
@@ -562,7 +580,7 @@ class _WireReader(object):
 
         if self.updating and qcount > 1:
             raise dns.exception.FormError
-        
+
         for i in xrange(0, qcount):
             (qname, used) = dns.name.from_wire(self.wire, self.current)
             if not self.message.origin is None:
@@ -577,7 +595,7 @@ class _WireReader(object):
                                     force_unique=True)
             if self.updating:
                 self.zone_rdclass = rdclass
-        
+
     def _get_section(self, section, count):
         """Read the next I{count} records from the wire data and add them to
         the specified section.
@@ -585,8 +603,8 @@ class _WireReader(object):
         @type section: list of dns.rrset.RRset objects
         @param count: the number of records to read
         @type count: int"""
-        
-        if self.updating:
+
+        if self.updating or self.one_rr_per_rrset:
             force_unique = True
         else:
             force_unique = False
@@ -594,6 +612,7 @@ class _WireReader(object):
         for i in xrange(0, count):
             rr_start = self.current
             (name, used) = dns.name.from_wire(self.wire, self.current)
+            absolute_name = name
             if not self.message.origin is None:
                 name = name.relativize(self.message.origin)
             self.current = self.current + used
@@ -607,6 +626,18 @@ class _WireReader(object):
                 self.message.payload = rdclass
                 self.message.ednsflags = ttl
                 self.message.edns = (ttl & 0xff0000) >> 16
+                self.message.options = []
+                current = self.current
+                optslen = rdlen
+                while optslen > 0:
+                    (otype, olen) = \
+                            struct.unpack('!HH',
+                                          self.wire[current:current + 4])
+                    current = current + 4
+                    opt = dns.edns.option_from_wire(otype, self.wire, current, olen)
+                    self.message.options.append(opt)
+                    current = current + olen
+                    optslen = optslen - 4 - olen
                 seen_opt = True
             elif rdtype == dns.rdatatype.TSIG:
                 if not (section is self.message.additional and
@@ -614,12 +645,12 @@ class _WireReader(object):
                     raise BadTSIG
                 if self.message.keyring is None:
                     raise UnknownTSIGKey, 'got signed message without keyring'
-                secret = self.message.keyring.get(name)
+                secret = self.message.keyring.get(absolute_name)
                 if secret is None:
                     raise UnknownTSIGKey, "key '%s' unknown" % name
                 self.message.tsig_ctx = \
-                	dns.tsig.validate(self.wire,
-                                          name,
+                                      dns.tsig.validate(self.wire,
+                                          absolute_name,
                                           secret,
                                           int(time.time()),
                                           self.message.request_mac,
@@ -640,7 +671,9 @@ class _WireReader(object):
                     rdclass = self.zone_rdclass
                 else:
                     deleting = None
-                if deleting == dns.rdataclass.ANY:
+                if deleting == dns.rdataclass.ANY or \
+                   (deleting == dns.rdataclass.NONE and \
+                    section == self.message.answer):
                     covers = dns.rdatatype.NONE
                     rd = None
                 else:
@@ -660,7 +693,7 @@ class _WireReader(object):
     def read(self):
         """Read a wire format DNS message and build a dns.message.Message
         object."""
-        
+
         l = len(self.wire)
         if l < 12:
             raise ShortHeader
@@ -684,7 +717,7 @@ class _WireReader(object):
 
 def from_wire(wire, keyring=None, request_mac='', xfr=False, origin=None,
               tsig_ctx = None, multi = False, first = True,
-              question_only = False):
+              question_only = False, one_rr_per_rrset = False):
     """Convert a DNS wire format message into a message
     object.
 
@@ -708,6 +741,8 @@ def from_wire(wire, keyring=None, request_mac='', xfr=False, origin=None,
     @type first: bool
     @param question_only: Read only up to the end of the question section?
     @type question_only: bool
+    @param one_rr_per_rrset: Put each RR into its own RRset
+    @type one_rr_per_rrset: bool
     @raises ShortHeader: The message is less than 12 octets long.
     @raises TrailingJunk: There were octets in the message past the end
     of the proper DNS message.
@@ -726,15 +761,15 @@ def from_wire(wire, keyring=None, request_mac='', xfr=False, origin=None,
     m.multi = multi
     m.first = first
 
-    reader = _WireReader(wire, m, question_only)
+    reader = _WireReader(wire, m, question_only, one_rr_per_rrset)
     reader.read()
 
     return m
-    
+
 
 class _TextReader(object):
     """Text format reader.
-    
+
     @ivar tok: the tokenizer
     @type tok: dns.tokenizer.Tokenizer object
     @ivar message: The message object being built
@@ -758,7 +793,7 @@ class _TextReader(object):
 
     def _header_line(self, section):
         """Process one line from the text format header section."""
-        
+
         (ttype, what) = self.tok.get()
         if what == 'id':
             self.message.id = self.tok.get_int()
@@ -803,7 +838,7 @@ class _TextReader(object):
 
     def _question_line(self, section):
         """Process one line from the text format question section."""
-        
+
         token = self.tok.get(want_leading = True)
         if token[0] != dns.tokenizer.WHITESPACE:
             self.last_name = dns.name.from_text(token[1], None)
@@ -834,7 +869,7 @@ class _TextReader(object):
         """Process one line from the text format answer, authority, or
         additional data sections.
         """
-        
+
         deleting = None
         # Name
         token = self.tok.get(want_leading = True)
@@ -913,7 +948,7 @@ class _TextReader(object):
                 continue
             self.tok.unget(token)
             line_method(section)
-        
+
 
 def from_text(text):
     """Convert the text format message into a message object.
@@ -927,14 +962,14 @@ def from_text(text):
     # 'text' can also be a file, but we don't publish that fact
     # since it's an implementation detail.  The official file
     # interface is from_file().
-    
+
     m = Message()
 
     reader = _TextReader(text, m)
     reader.read()
-    
+
     return m
-    
+
 def from_file(f):
     """Read the next text format message from the specified file.
 
@@ -973,7 +1008,7 @@ def make_query(qname, rdtype, rdclass = dns.rdataclass.IN, use_edns=None,
 
     The query will have a randomly choosen query id, and its DNS flags
     will be set to dns.flags.RD.
-    
+
     @param qname: The query name.
     @type qname: dns.name.Name object or string
     @param rdtype: The desired rdata type.
@@ -987,7 +1022,7 @@ def make_query(qname, rdtype, rdclass = dns.rdataclass.IN, use_edns=None,
     @param want_dnssec: Should the query indicate that DNSSEC is desired?
     @type want_dnssec: bool
     @rtype: dns.message.Message object"""
-    
+
     if isinstance(qname, (str, unicode)):
         qname = dns.name.from_text(qname)
     if isinstance(rdtype, str):
@@ -1011,7 +1046,7 @@ def make_response(query, recursion_available=False, our_payload=8192):
     The response's question section is a shallow copy of the query's
     question section, so the query's question RRsets should not be
     changed.
-    
+
     @param query: the query to respond to
     @type query: dns.message.Message object
     @param recursion_available: should RA be set in the response?
