@@ -1,4 +1,4 @@
-# Copyright (C) 2003-2005 Nominum, Inc.
+# Copyright (C) 2003-2007, 2009 Nominum, Inc.
 #
 # Permission to use, copy, modify, and distribute this software and its
 # documentation for any purpose with or without fee is hereby granted,
@@ -44,10 +44,8 @@ class Update(dns.message.Message):
         """
         super(Update, self).__init__()
         self.flags |= dns.opcode.to_flags(dns.opcode.UPDATE)
-        if isinstance(zone, str):
+        if isinstance(zone, (str, unicode)):
             zone = dns.name.from_text(zone)
-        else:
-            zone = zone.copy()
         self.origin = zone
         if isinstance(rdclass, str):
             rdclass = dns.rdataclass.from_text(rdclass)
@@ -80,7 +78,7 @@ class Update(dns.message.Message):
 
                 - ttl, rdtype, string..."""
 
-        if isinstance(name, str):
+        if isinstance(name, (str, unicode)):
             name = dns.name.from_text(name, None)
         if isinstance(args[0], dns.rdataset.Rdataset):
             for rds in args:
@@ -130,7 +128,7 @@ class Update(dns.message.Message):
 
                 - rdtype, [string...]"""
 
-        if isinstance(name, str):
+        if isinstance(name, (str, unicode)):
             name = dns.name.from_text(name, None)
         if len(args) == 0:
             rrset = self.find_rrset(self.authority, name, dns.rdataclass.ANY,
@@ -188,7 +186,7 @@ class Update(dns.message.Message):
 
                 - rdtype, string..."""
         
-        if isinstance(name, str):
+        if isinstance(name, (str, unicode)):
             name = dns.name.from_text(name, None)
         if len(args) == 0:
             rrset = self.find_rrset(self.answer, name,
@@ -216,7 +214,7 @@ class Update(dns.message.Message):
         """Require that an owner name (and optionally an rdata type) does
         not exist as a prerequisite to the execution of the update."""
 
-        if isinstance(name, str):
+        if isinstance(name, (str, unicode)):
             name = dns.name.from_text(name, None)
         if rdtype is None:
             rrset = self.find_rrset(self.answer, name,
